searchNodes=[{"doc":"Unpickler is a library for loading data in the Python's pickle format. Supports all pickle protocols from 0 to 5.","ref":"Unpickler.html","title":"Unpickler","type":"module"},{"doc":"Loads the given pickle binary. Basic literals and data structures are deserialized as corresponding Elixir terms whenever possible. The pickle data may include arbitrary Python objects, so in all other cases the Unpickler.Object struct is used. This struct holds all the information that would be used for object reconstruction. You can define a custom :object_resolver function to recognise certain objects and map them to whatever data structure you see fit. Object references Note that if the object hierarchy includes circular references, it is inherently impossible to represent in Elixir. One example of such data structure is a list with a reference to itself: x = [] x.append(x) On the other hand, multiple references to the same object are restored as expected, without duplicating memory, as in: x = [1, 2, 3] y = (x, x) Options :object_resolver - a function for constructing a custom term corresponding to a Python object. Receives Unpickler.Object as an argument and should return the term as {:ok, term} or :error if not applicable :persistent_id_resolver - a function returning an object for the given persistent id. This function is required if the data includes a persistent id, otherwise an error is raised Examples The scalar 1 would be loaded like so: iex&gt; Unpickler . load! ( &lt;&lt; 128 , 4 , 75 , 1 , 46 &gt;&gt; ) { 1 , &quot;&quot; } Next, a more complex data structure: [1, 2.0, &quot;text&quot;, (None, True), {&quot;key&quot;: &quot;val&quot;}, b&quot;\\x01\\x00&quot;] iex&gt; data = ...&gt; &lt;&lt; 128 , 4 , 149 , 47 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 93 , 148 , 40 , 75 , 1 , 71 , 64 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , ...&gt; 140 , 4 , 116 , 101 , 120 , 116 , 148 , 78 , 136 , 134 , 148 , 125 , 148 , 140 , 3 , 107 , 101 , 121 , ...&gt; 148 , 140 , 3 , 118 , 97 , 108 , 148 , 115 , 67 , 2 , 1 , 0 , 148 , 101 , 46 &gt;&gt; iex&gt; Unpickler . load! ( data ) { [ 1 , 2.0 , &quot;text&quot; , { nil , true } , %{ &quot;key&quot; =&gt; &quot;val&quot; } , &lt;&lt; 1 , 0 &gt;&gt; ] , &quot;&quot; } Objects Other objects end up as Unpickler.Object from datetime import date date.fromisoformat(&quot;2022-05-17&quot;) iex&gt; data = ...&gt; &lt;&lt; 128 , 4 , 149 , 32 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 140 , 8 , 100 , 97 , 116 , 101 , 116 , 105 , 109 , 101 , ...&gt; 148 , 140 , 4 , 100 , 97 , 116 , 101 , 148 , 147 , 148 , 67 , 4 , 7 , 230 , 5 , 17 , 148 , 133 , 148 , 82 , ...&gt; 148 , 46 &gt;&gt; iex&gt; Unpickler . load! ( data ) { % Unpickler.Object { append_items : [ ] , args : [ &lt;&lt; 7 , 230 , 5 , 17 &gt;&gt; ] , constructor : &quot;datetime.date&quot; , kwargs : %{ } , set_items : [ ] , state : nil } , &quot;&quot; } For those, we can customize construction by specifying an :object_resolver from datetime import date date.fromisoformat(&quot;2022-05-17&quot;) iex&gt; data = ...&gt; &lt;&lt; 128 , 4 , 149 , 32 , 0 , 0 , 0 , 0 , 0 , 0 , 0 , 140 , 8 , 100 , 97 , 116 , 101 , 116 , 105 , 109 , 101 , ...&gt; 148 , 140 , 4 , 100 , 97 , 116 , 101 , 148 , 147 , 148 , 67 , 4 , 7 , 230 , 5 , 17 , 148 , 133 , 148 , 82 , ...&gt; 148 , 46 &gt;&gt; iex&gt; object_resolver = fn ...&gt; # See https://github.com/python/cpython/blob/3.10/Lib/datetime.py#L1094-L1105 ...&gt; %{ constructor : &quot;datetime.date&quot; , args : [ &lt;&lt; year_hi , year_lo , month , day &gt;&gt; ] } -&gt; ...&gt; { :ok , date } = Date . new ( year_hi * 256 + year_lo , month , day ) ...&gt; { :ok , date } ...&gt; ...&gt; _ -&gt; ...&gt; :error ...&gt; end iex&gt; Unpickler . load! ( data , object_resolver : object_resolver ) { ~D[2022-05-17] , &quot;&quot; }","ref":"Unpickler.html#load!/2","title":"Unpickler.load!/2","type":"function"},{"doc":"Represents a reference to a global class or function.","ref":"Unpickler.Global.html","title":"Unpickler.Global","type":"module"},{"doc":"Represents a Python object. This struct includes all the information that would be used for object reconstruction. Refer to the pickle documentation for more details. Information The following fields are available: :constructor - name of the class or function called to build the object :args - a list of arguments passed to the constructor :kwargs - a map with keyword arguments passed to the constructor :state - a value passed to the __setstate__ method if applicable :append_items - a list of values to append to the object :set_items - a list of key-value pairs to set on the object","ref":"Unpickler.Object.html","title":"Unpickler.Object","type":"module"},{"doc":"Mix . install ( [ { :unpickler , &quot;~&gt; 0.1.0&quot; } , { :nx , &quot;~&gt; 0.4.0&quot; } ] ) :ok","ref":"examples.html","title":"Examples","type":"extras"},{"doc":"We will first define a helper macro to automatically generate a pickle binary from example Python code. defmodule Pickle do @doc &quot;&quot;&quot; Runs the given Python code and returns the pickle binary. The code must define a variable `x` and that value is pickled. Optionally, a numeric sigil modifier may be given indicating the pickle version to use, defaults to version 4. Note that a `python` executable must be available. ## Examples iex&gt; import Pickle iex&gt; ~P&#39;&#39;&#39; ...&gt; x = 1 ...&gt; &#39;&#39;&#39; &lt;&lt;128, 4, 75, 1, 46&gt;&gt; &quot;&quot;&quot; defmacro sigil_P ( { :&lt;&lt;&gt;&gt; , _meta , [ code ] } , modifiers ) do protocol = case modifiers do [ ] -&gt; 4 [ version ] -&gt; version - ?0 end code = &quot;&quot;&quot; import pickle import base64 \#{ code } data = pickle.dumps(x, protocol= \#{ protocol } ) encoded = base64.b32encode(data).decode(&quot;utf-8&quot;) print(encoded, end=&quot;&quot;) &quot;&quot;&quot; { encoded , 0 } = System . cmd ( &quot;python&quot; , [ &quot;-c&quot; , code ] ) binary = Base . decode32! ( encoded ) Macro . escape ( binary ) end end 1 doctest , 0 failures { :module , Pickle , &lt;&lt; 70 , 79 , 82 , 49 , 0 , 0 , 11 , ... &gt;&gt; , { :sigil_P , 2 } } import Pickle Pickle","ref":"examples.html#setup","title":"Examples - Setup","type":"extras"},{"doc":"~P&quot;&quot;&quot; x = 1 &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { 1 , &quot;&quot; } ~P&quot;&quot;&quot; x = [1, 2, 3] &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { [ 1 , 2 , 3 ] , &quot;&quot; } ~P&quot;&quot;&quot; x = [1, 2.0, &quot;text&quot;, (None, True), {&quot;key&quot;: &quot;val&quot;}, b&quot;\\x01\\x00&quot;] &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { [ 1 , 2.0 , &quot;text&quot; , { nil , true } , %{ &quot;key&quot; =&gt; &quot;val&quot; } , &lt;&lt; 1 , 0 &gt;&gt; ] , &quot;&quot; }","ref":"examples.html#basic-data-structures","title":"Examples - Basic data structures","type":"extras"},{"doc":"All non-trivial objects are deserialized as %Unpickler.Object{} structs. For example let's consider a date object: ~P&quot;&quot;&quot; from datetime import date x = date.fromisoformat(&quot;2022-05-17&quot;) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;datetime.date&quot; , args : [ &lt;&lt; 7 , 230 , 5 , 17 &gt;&gt; ] , kwargs : %{ } , state : nil , append_items : [ ] , set_items : [ ] } , &quot;&quot; } However, ideally we would want to load that object as Elixir %Date{} . To do that, we can specify an object resolver: object_resolver = fn # See https://github.com/python/cpython/blob/3.10/Lib/datetime.py#L1094-L1105 % Unpickler.Object { constructor : &quot;datetime.date&quot; , args : [ &lt;&lt; year_hi , year_lo , month , day &gt;&gt; ] } -&gt; { :ok , date } = Date . new ( year_hi * 256 + year_lo , month , day ) { :ok , date } _ -&gt; :error end ~P&quot;&quot;&quot; from datetime import date x = date.fromisoformat(&quot;2022-05-17&quot;) &quot;&quot;&quot; |&gt; Unpickler . load! ( object_resolver : object_resolver ) { ~D[2022-05-17] , &quot;&quot; } Note that %Unpickler.Object{} has a number of different fields, that's because Python objects may define multiple ways of serializing themselves and we gather all the information under a single unified struct. This way we can easily pattern match on it in the object resolver. When deserializing complex objects you generally need to look at the corresponding Python source code and reverse-engineer how to load them accordingly. Here are a couple examples: ~P&quot;&quot;&quot; class Point: def __init__(self, x, y): self.x = x self.y = y x = Point(1, 1) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;__main__.Point.__new__&quot; , args : [ % Unpickler.Global { scope : &quot;__main__&quot; , name : &quot;Point&quot; } ] , kwargs : %{ } , state : %{ &quot;x&quot; =&gt; 1 , &quot;y&quot; =&gt; 1 } , append_items : [ ] , set_items : [ ] } , &quot;&quot; } ~P&quot;&quot;&quot; class Point: def __init__(self, x, y): self.x = x self.y = y def __reduce__(self): return (Point, (self.x, self.y)) x = Point(1, 1) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;__main__.Point&quot; , args : [ 1 , 1 ] , kwargs : %{ } , state : nil , append_items : [ ] , set_items : [ ] } , &quot;&quot; } ~P&quot;&quot;&quot; class Point: def __init__(self, x, y): self.x = x self.y = y @classmethod def _reconstruct(cls, x, y): return cls(x, y) def __reduce__(self): return (Point._reconstruct, (self.x, self.y)) x = Point(1, 1) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;__main__.Point._reconstruct&quot; , args : [ 1 , 1 ] , kwargs : %{ } , state : nil , append_items : [ ] , set_items : [ ] } , &quot;&quot; } ~P&quot;&quot;&quot; class Point: def __init__(self, x, y): self.x = x self.y = y def __getstate__(self): return (self.x, self.y) def __setstate__(self, state): (x, y) = state self.x = x self.y = y x = Point(1, 1) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;__main__.Point.__new__&quot; , args : [ % Unpickler.Global { scope : &quot;__main__&quot; , name : &quot;Point&quot; } ] , kwargs : %{ } , state : { 1 , 1 } , append_items : [ ] , set_items : [ ] } , &quot;&quot; } ~P&quot;&quot;&quot; class Point: def __init__(self, x, y): self.x = x self.y = y def __reduce__(self): append_iter = iter([1, 2]) set_iter = iter([(&quot;a&quot;, 1), (&quot;b&quot;, 2)]) return (Point, (self.x, self.y), None, append_iter, set_iter) x = Point(1, 1) &quot;&quot;&quot; |&gt; Unpickler . load! ( ) { % Unpickler.Object { constructor : &quot;__main__.Point&quot; , args : [ 1 , 1 ] , kwargs : %{ } , state : nil , append_items : [ 1 , 2 ] , set_items : [ { &quot;a&quot; , 1 } , { &quot;b&quot; , 2 } ] } , &quot;&quot; }","ref":"examples.html#custom-object-resolver","title":"Examples - Custom object resolver","type":"extras"}]